# -*- coding: utf-8 -*-
# Copyright (C) 2010-2014 Martin Glueck All rights reserved
# Langstrasse 4, A--2244 Spannberg, Austria. martin@mangari.org
# ****************************************************************************
# This module is part of the package JNJ.
#
# This module is licensed under the terms of the BSD 3-Clause License
# <http://www.c-tanzer.at/license/bsd_3c.html>.
# ****************************************************************************
#
#++
# Name
#    JNJ.Babel
#
# Purpose
#    Add additional keywords to the default extraction method and adopt the
#    parameter's
#    Heavily inspired by the jinja2.ext module (unfortunately we had to copy
#    the code to make it work for your purposes)
#
# Revision Dates
#    24-Jan-2010 (MG) Creation
#    25-Jan-2010 (MG) Looks like we need to strip the first new line in order
#                     to have the right key for the lookup
#    23-Feb-2010 (MG) `babel_extract` copied over from `jinja2.ext` because
#                     we want to see the `TemplateSyntaxError`s and we need
#                     to add the `JNJ.Environment.Default_Extensions` to the
#                     environment
#    26-Nov-2010 (CT) Import for `get_spontaneous_environment` fixed
#     4-Dec-2013 (CT) Adapt `babel_extract` to jinja2 version 2.7
#    ««revision-date»»···
#--

from   _JNJ                import JNJ
import _JNJ.Environment

from   _TFL._Babel.Extract import Default_Keywords
from   _TFL.pyk            import pyk

import jinja2
from   jinja2              import ext

def Extract (fobj, keywords, comment_tags, config, method) :
    keywords = Default_Keywords
    for lineno, funcname, messages, comments in babel_extract \
        (fobj, keywords, comment_tags, config.options (method)) :
        yield lineno, funcname, messages, comments, None
# end def Extract

if jinja2.__version__ >= "2.7" :
    def babel_extract (fileobj, keywords, comment_tags, options) :
        """Babel extraction method for Jinja templates.

        :param fileobj: the file-like object the messages should be extracted from
        :param keywords: a list of keywords (i.e. function names) that should be
                         recognized as translation functions
        :param comment_tags: a list of translator tags to search for and include
                             in the results.  (Unused)
        :param options: a dictionary of additional options (optional)
        :return: an iterator over ``(lineno, funcname, message, comments)`` tuples.
                 (comments will be empty currently)
        """
        extensions = set ()
        for extension in options.get ("extensions", "").split (",") :
            extension = extension .strip ()
            if not extension:
                continue
            extensions.add (ext.import_string (extension))
        for extension in JNJ.Environment.Default_Extensions :
            if isinstance (extension, pyk.string_types) :
                extension = ext.import_string (extension)
            extensions.add (extension)
        if ext.InternationalizationExtension not in extensions :
            extensions.add (ext.InternationalizationExtension)

        def getbool(options, key, default=False):
            return options.get (key, str (default)).lower () in \
                ('1', 'on', 'yes', 'true')

        environment = ext.Environment \
            ( options.get ("block_start_string",    ext.BLOCK_START_STRING)
            , options.get ("block_end_string",      ext.BLOCK_END_STRING)
            , options.get ("variable_start_string", ext.VARIABLE_START_STRING)
            , options.get ("variable_end_string",   ext.VARIABLE_END_STRING)
            , options.get ("comment_start_string",  ext.COMMENT_START_STRING)
            , options.get ("comment_end_string",    ext.COMMENT_END_STRING)
            , options.get ("line_statement_prefix") or ext.LINE_STATEMENT_PREFIX
            , options.get ("line_comment_prefix") or ext.LINE_COMMENT_PREFIX
            , getbool (options, "trim_blocks",   ext.TRIM_BLOCKS)
            , getbool (options, "lstrip_blocks", ext.LSTRIP_BLOCKS)
            , ext.NEWLINE_SEQUENCE
            , getbool (options, "keep_trailing_newline", ext.KEEP_TRAILING_NEWLINE)
            , frozenset (extensions)
            , cache_size  = 0
            , auto_reload = False
            )

        source = fileobj.read ().decode (options.get ("encoding", "utf-8"))
        node   = environment.parse (source)
        for lineno, func, message in extract_from_ast (node, keywords) :
            yield lineno, func, message, []
    # end def babel_extract
else :
    from jinja2.environment import get_spontaneous_environment
    def babel_extract (fileobj, keywords, comment_tags, options) :
        """Babel extraction method for Jinja templates.

        :param fileobj: the file-like object the messages should be extracted from
        :param keywords: a list of keywords (i.e. function names) that should be
                         recognized as translation functions
        :param comment_tags: a list of translator tags to search for and include
                             in the results.  (Unused)
        :param options: a dictionary of additional options (optional)
        :return: an iterator over ``(lineno, funcname, message, comments)`` tuples.
                 (comments will be empty currently)
        """
        extensions = set ()
        for extension in options.get ("extensions", "").split (",") :
            extension = extension .strip ()
            if not extension:
                continue
            extensions.add (ext.import_string (extension))
        for extension in JNJ.Environment.Default_Extensions :
            if isinstance (extension, pyk.string_types) :
                extension = ext.import_string (extension)
            extensions.add (extension)
        if ext.InternationalizationExtension not in extensions :
            extensions.add (ext.InternationalizationExtension)

        environment = get_spontaneous_environment \
            ( options.get ("block_start_string",    ext.BLOCK_START_STRING)
            , options.get ("block_end_string",      ext.BLOCK_END_STRING)
            , options.get ("variable_start_string", ext.VARIABLE_START_STRING)
            , options.get ("variable_end_string",   ext.VARIABLE_END_STRING)
            , options.get ("comment_start_string",  ext.COMMENT_START_STRING)
            , options.get ("comment_end_string",    ext.COMMENT_END_STRING)
            , options.get ("line_statement_prefix") or ext.LINE_STATEMENT_PREFIX
            , options.get ("line_comment_prefix") or ext.LINE_COMMENT_PREFIX
            , str (options.get ("trim_blocks", ext.TRIM_BLOCKS)).lower()
              in ("1", "on", "yes", "true")
            , ext.NEWLINE_SEQUENCE
            , frozenset (extensions)
            # fill with defaults so that environments are shared
            # with other spontaneus environments.  The rest of the
            # arguments are optimizer, undefined, finalize, autoescape,
            # loader, cache size, auto reloading setting and the
            # bytecode cache
            , True, ext.Undefined, None, False, None, 0, False, None
            )

        source = fileobj.read ().decode (options.get ("encoding", "utf-8"))
        node   = environment.parse (source)
        for lineno, func, message in extract_from_ast (node, keywords) :
            yield lineno, func, message, []
    # end def babel_extract

def extract_from_ast ( node
                     , gettext_functions = ext.GETTEXT_FUNCTIONS
                     , babel_style       = True
                     ) :
    """Copied from jinja2.ext because we have translation calls hidden in
       Getattr nodes as well.
    """
    for node in node.find_all (ext.nodes.Call) :
        if (  (  not isinstance (node.node, ext.nodes.Name)
              or node.node.name not in gettext_functions
              )
          and ( not isinstance (node.node, ext.nodes.Getattr)
              or node.node.attr not in gettext_functions
              )
          ) :
            continue

        strings = []
        for arg in node.args :
            if (   isinstance (arg,       ext.nodes.Const)
               and isinstance (arg.value, pyk.string_types)
               ) :
                strings.append (arg.value.lstrip ("\n"))
            else:
                strings.append (None)

        for arg in node.kwargs :
            strings.append (None)
        if node.dyn_args is not None :
            strings.append (None)
        if node.dyn_kwargs is not None :
            strings.append (None)

        if not babel_style :
            strings = tuple(x for x in strings if x is not None)
            if not strings :
                continue
        else:
            if len (strings) == 1 :
                strings = strings [0]
            else:
                strings = tuple (strings)

        if isinstance (node.node, ext.nodes.Name) :
            yield node.lineno, node.node.name, strings
        else :
            yield node.lineno, node.node.attr, strings
# end def extract_from_ast

### __END__ JNJ.Babel



# -*- coding: utf-8 -*-
# Copyright (C) 2010-2016 Mag. Christian Tanzer All rights reserved
# Glasauergasse 32, A--1130 Wien, Austria. tanzer@swing.co.at
# ****************************************************************************
# This module is part of the package JNJ.
#
# This module is licensed under the terms of the BSD 3-Clause License
# <http://www.c-tanzer.at/license/bsd_3c.html>.
# ****************************************************************************
#
#++
# Name
#    JNJ.Templateer
#
# Purpose
#    Encapsulate Jinja template handling
#
# Revision Dates
#    14-Jan-2010 (CT) Creation
#    15-Jan-2010 (CT) `Error_Templates` added
#    18-Jan-2010 (CT) `get_std_template` added; s/Error_Templates/Template_Map/
#    29-Jan-2010 (CT) `get_template` changed to use `Template_Map`
#    19-Feb-2010 (CT) `error_405` added
#    14-May-2010 (MG) `dynamic_form` added
#    19-May-2010 (MG) `render_string` added
#     2-Aug-2010 (MG) `console` added
#    17-Aug-2010 (CT) `error_503` added
#    29-Dec-2010 (CT) `Template` added and used
#    31-Dec-2010 (CT) `Template_E` added and used
#     2-Jan-2011 (CT) `Template_E.get_CSS` added
#     3-Jan-2011 (CT) `Template_E.CSS` added
#     3-Jan-2011 (CT) `Template_E.get_CSS` changed to sort stylesheets by `rank`
#     4-Jan-2011 (CT) `templates_i` fixed (transitive `imports`)
#     5-Jan-2011 (CT) `templates_i` fixed (protect against recursive `imports`)
#     5-Jan-2011 (CT) `Template_E.__eq__` and `__hash__` added
#    14-Jan-2011 (CT) `get_CSS` replaced by `get_Media` (s/css/media/ -- mostly)
#    14-Jan-2011 (CT) `JS`, `JS_On_Ready`, `js_href`, `js_on_ready` and
#                     `scripts` added
#    16-Mar-2011 (CT) `Template_E.module`, `.get_macro`, and `.call_macro` added
#    16-Mar-2011 (CT) `Templateer.__init__` changed to pass `GTW` to
#                     `JNJ.Environment.HTML` -> `GTW.env` refers to `Templateer`
#    17-Mar-2011 (CT) `afs` and `e_type_afs` added
#    18-Mar-2011 (CT) `Template_E.parent` added and used in `get_macro`
#    31-May-2011 (CT) `render` changed to put `template` into `context`
#    15-Jun-2011 (MG) `get_css` added, `get_Media` and `_eval_fragments`
#                     changed to `classmethod`
#    27-Sep-2011 (MG) `s/CSS_Links/css_links/`, `s/JS_On_Ready/js_on_ready/`
#                     `js_on_ready` return the objects instead of the
#                     combined code
#                     `_eval_fragments`: use `scope.Eval`
#    24-Nov-2011 (CT) Add `Templateer.call_macro`
#     1-Dec-2011 (CT) Add `error_templates` and `error_template_names`
#    14-Dec-2011 (CT) Add `rel_links`
#     3-Jan-2012 (CT) Fix `js`
#     5-Jan-2012 (CT) Add `get_cached_media`, `Media_Map`, `_Media_[CR]`
#     9-Jan-2012 (CT) Fix `get_cached_media` (use `.scripts` unless `js_href`)
#    27-Jan-2012 (CT) Fix `__eq__` and `__hash__`,
#                     add `__lt__` and `totally_ordered`
#    29-Mar-2012 (CT) Rename `CSS_Parameters` to `Media_Parameters`
#    15-Apr-2012 (CT) Apply `sorted` to `injected` in `Templateer.get_template`
#    30-Apr-2012 (CT) Add `regatta_register_email`
#    10-May-2012 (CT) Add `error_email`
#     7-Jun-2012 (CT) Use `TFL.r_eval`, factor `env_globals`
#    22-Jun-2012 (CT) Use `static_handler.get_path` instead of home-grown code
#     6-Aug-2012 (CT) Add `etag`
#     6-Aug-2012 (CT) Remove `e_type_aggregator` (template doesn't exist)
#     8-Aug-2012 (CT) Add `e_type_doc`
#    25-Sep-2012 (CT) Add `pns_e_type_doc`
#     9-Oct-2012 (CT) Remove `account_activate`
#    11-Dec-2012 (CT) Add `error_410`
#     8-Jan-2013 (CT) Add `account_make_cert`
#    22-Jan-2014 (CT) Add `empty_dir`
#    12-Feb-2014 (CT) Add `regatta_ranking`
#    11-Mar-2014 (CT) Add `e_type_display`
#    15-Apr-2014 (CT) Add `media.script_files` to `Template_E.js`
#    24-Apr-2014 (CT) Add `afs_input`
#    24-Apr-2014 (CT) Add `mf3`, `mf3_input`, `mf3_v_seq`, `mf3_h_cols`
#     9-Jul-2014 (CT) Changge `js` to use `pyk.decoded`, not home-grown code,
#                     to decode entries of `media.script_files`
#    29-Aug-2014 (CT) Remove `AFS` specific templates
#    12-Oct-2014 (CT) Add `sk` to `__lt__` to avoid Python-3 exception
#                     because different types
#    21-Jan-2015 (CT) Add `ETR_table`
#    11-Feb-2015 (CT) Factor `SRM` template declarations,
#                     remove obsolete templates
#    12-Jun-2015 (CT) Add `account_change_email_info`,
#                     `account_change_password_info`
#     2-Jun-2016 (CT) Add `e_type_selector`
#    11-Oct-2016 (CT) Import from `CHJ`, not `GTW`
#    ««revision-date»»···
#--

from   __future__  import unicode_literals

from   _JNJ               import JNJ
from   _TFL               import TFL
from   _TFL.pyk           import pyk

import _JNJ.Environment
import _JNJ.GTW

import _TFL._Meta.Object
import _TFL.Accessor
import _TFL.multimap
import _TFL.predicate
import _TFL.r_eval
import _TFL.Sorted_By

from   _TFL._Meta.Once_Property   import Once_Property
from   _TFL._Meta.totally_ordered import totally_ordered
from   _TFL.Regexp                import *

from   jinja2.exceptions import TemplateNotFound

class _Template_ (TFL.Meta.Object) :
    """Describe a Jinja template."""

    def _init_ (self, name, path, media_fragment_name = None, parent_name = None) :
        assert name not in self.Map, name
        self.name                = name
        self.path                = path
        self.media_fragment_name = media_fragment_name
        self.parent_name         = parent_name
        self.id                  = len (self.Map)
        self.Map [name]          = self
    # end def _init_

    @property
    def args (self) :
        return self.name, self.path, self.media_fragment_name, self.parent_name
    # end def args

# end class _Template_

class Template (_Template_) :
    """Describe a Jinja template."""

    Map           = {}

    __init__      = _Template_._init_

# end class Template

@totally_ordered
class Template_E (_Template_) :
    """Describe a Jinja template for a specific Jinja environment."""

    Media_Map       = {}
    css_href_map    = {}

    _media_fragment = None
    _media_path     = None

    _coding_pat     = Regexp \
        ( r"^#\s*-\*-\s*coding:\s*[-a-zA-Z0-9]+\s*-\*-\s*" + "\n"
        )
    _extend_pat     = Regexp \
        ( r"\{\%-?\s*extends\s+(?P<name>.*?)\s*-?\%\}"
        )
    _import_pat     = Multi_Regexp \
        ( r"\{\%-?\s*import\s+"
          r"(?P<name>.*?)\s+"
          r"as\s+"
        , r"\{\%-?\s*from\s+"
          r"(?P<name>.*?)\s+"
          r"import\s+"
        , r"\{\%-\s*include\s+"
          r"(?P<name>.*?)\s+"
          r"(?:(?:ignore|with|without)\s+|-?\%\})"
        )

    _t_path         = None
    _t_source       = None

    def __new__ (cls, env, name, path = None, * args, ** kw) :
        if path in cls.By_Path :
            result = cls.By_Path [path]
        elif name in cls.Map :
            result = cls.Map     [name]
        else :
            if path is None :
                path = name
            result = _Template_.__new__ (cls)
            result._init_ (env, name, path, * args, ** kw)
        return result
    # end def __new__

    def _init_ (self, env, * args, ** kw) :
        self.env      = env
        self.injected = kw.pop ("injected", ())
        self._macros  = {}
        self.__super._init_ (* args, ** kw)
        if self.path not in self.By_Path :
            self.By_Path [self.path] = self
    # end def _init_

    @classmethod
    def copy (cls, env, proto) :
        return cls (env, * proto.args)
    # end def copy

    @Once_Property
    def CSS (self) :
        """Combined CSS rules required by media fragments that can
           loaded from a single file or included inline in a html <style>
           element.
        """
        return self.get_css (self._Media_R)
    # end def CSS

    @property
    def css_href (self) :
        return self.css_href_map.get (self.name)
    # end def css_href

    @Once_Property
    def css_links (self) :
        media = self._Media
        if media :
            return sorted (media.css_links, key = TFL.Getter.rank)
        return ()
    # end def css_links

    @Once_Property
    def env_globals (self) :
        return dict \
            (  (k, v) for k, v in pyk.iteritems (self.env.globals)
            if not k.startswith ("_")
            )
    # end def env_globals

    @Once_Property
    def extends (self) :
        env    = self.env
        pat    = self._extend_pat
        source = self.source
        if source and pat.search (source) :
            try :
                path = TFL.r_eval (pat.name.strip (), ** self.env_globals)
            except Exception :
                pass
            else :
                return self.__class__ (env, path)
    # end def extends

    @Once_Property
    def imports (self) :
        env    = self.env
        pat    = self._import_pat
        source = self.source
        if source :
            def _gen () :
                for match in pat.search_iter (source) :
                    try:
                        name   = match.group ("name").strip ()
                        pathes = TFL.r_eval  (name, ** self.env_globals)
                    except Exception :
                        pass
                    else :
                        if isinstance (pathes, pyk.string_types) :
                            pathes = [pathes]
                        for p in pathes :
                            yield self.__class__ (env, p)
            return list (_gen ())
        return []
    # end def imports

    @Once_Property
    def js (self) :
        """Combined Javascript code required by media fragments that can be
           loaded from a single file or included inline in a html <script>
           element.
        """
        encoding = self.env.encoding
        handler  = self.env.static_handler
        media    = self._Media_R
        if handler and media :
            def _gen (handler, encoding, scripts, media) :
                for s in sorted (scripts, key = TFL.Getter.rank) :
                    p = handler.get_path (s.src)
                    if p :
                        with open (p, "rb") as file :
                            yield file.read ().decode (encoding)
                for s in sorted (media.script_files, key = TFL.Getter.rank) :
                    yield pyk.decoded (s.body, encoding)
            result = "\n\n".join \
                (TFL.uniq (_gen (handler, encoding, self.scripts_c, media)))
            return result
    # end def js

    @Once_Property
    def js_on_ready (self) :
        """Combined Javascript code required by media fragments to be
           executed when document is ready.
        """
        media = self._Media
        if media :
            return sorted (TFL.uniq (media.js_on_ready), key = TFL.Getter.rank)
        return ()
    # end def js_on_ready

    @Once_Property
    def media_fragment (self) :
        self._load_media ()
        return self._media_fragment
    # end def media_fragment

    @Once_Property
    def media_path (self) :
        self._load_media ()
        return self._media_path
    # end def media_path

    @Once_Property
    def module (self) :
        return self.template.module
    # end def module

    @Once_Property
    def parent (self) :
        if self.parent_name :
            return self.Map [self.parent_name]
    # end def parent

    @Once_Property
    def rel_links (self) :
        media = self._Media
        if media :
            return sorted (TFL.uniq (media.rel_links), key = TFL.Getter.rank)
        return ()
    # end def rel_links

    @Once_Property
    def scripts (self) :
        """Scripts required by media fragments that need to be put into
           separate <script src="..."> elements.
        """
        media = self._Media
        if media :
            return tuple (TFL.uniq (media.scripts))
        return ()
    # end def scripts

    @Once_Property
    def scripts_c (self) :
        """Scripts required by media fragments that can be cached."""
        media = self._Media_R
        if media :
            return tuple (s for s in TFL.uniq (media.scripts) if s.cache_p)
        return ()
    # end def scripts_c

    @Once_Property
    def scripts_x (self) :
        """Scripts required by media fragments that can not be cached."""
        media = self._Media_R
        if media :
            return tuple (s for s in TFL.uniq (media.scripts) if not s.cache_p)
        return ()
    # end def scripts_x

    @Once_Property
    def source (self) :
        self._load_template ()
        return self._t_source
    # end def source

    @Once_Property
    def source_path (self) :
        self._load_template ()
        return self._t_path
    # end def source_path

    @property
    def template (self) :
        return self.env.get_template (self.path)
    # end def template

    @Once_Property
    def templates (self) :
        return tuple (reversed (self.templates_e)) + self.templates_i
    # end def templates

    @Once_Property
    def templates_e (self) :
        def _gen () :
            yield self
            if self.extends :
                for e in self.extends.templates_e :
                    yield e
        return tuple (TFL.uniq (_gen ()))
    # end def templates_e

    @Once_Property
    def templates_i (self) :
        seen = set ([self])
        def _gen_i (imports) :
            for i in imports :
                if i not in seen :
                    yield i
                    seen.add (i)
                    for ii in _gen_i (i.imports) :
                        yield ii
        def _gen () :
            for e in reversed (self.templates_e) :
                for i in _gen_i (e.imports) :
                    yield i
            for i in _gen_i (self.injected) :
                yield i
        return tuple (_gen ())
    # end def templates_i

    @property
    def _Media (self) :
        result = self._Media_C
        if result is None :
            result = self._Media_R
        return result
    # end def _Media

    @property
    def _Media_C (self) :
        return self.Media_Map.get (self.name)
    # end def _Media_C

    @Once_Property
    def _Media_R (self) :
        return self.get_Media_R (self.env, self.templates)
    # end def _Media_R

    def call_macro (self, name, * _args, ** _kw) :
        macro = self.get_macro (name)
        return macro (* _args, ** _kw)
    # end def call_macro

    def get_cached_media (self, css_href, js_href) :
        from _CHJ.Media import Media
        if js_href :
            scripts = self.scripts_x + (js_href, )
        else :
            scripts = self.scripts
        result = Media \
            ( self.css_links
            , scripts
            , ("\n".join (pyk.decoded (x) for x in self.js_on_ready), )
            , self.rel_links
            )
        if css_href :
            self.css_href_map [self.name] = css_href
        return result
    # end def get_cached_media

    @classmethod
    def get_css (cls, media) :
        if media :
            return "\n\n".join \
                ( pyk.decoded (s) for s in
                    sorted (media.style_sheets, key = TFL.Getter.rank)
                )
    # end def get_css

    def get_macro (self, name) :
        result = self._macros.get (name)
        if result is None :
            result = getattr (self.module, name, None)
            if result is None :
                if self.parent :
                    result = self.parent.get_macro (name)
                else :
                    raise AttributeError (name)
            self._macros [name] = result
        return result
    # end def get_macro

    @classmethod
    def get_Media_R (cls, env, templates) :
        P = env.Media_Parameters
        media_fragment_pathes = tuple \
            (TFL.uniq (t.media_path for t in templates if t.media_path))
        if media_fragment_pathes :
            return cls._eval_fragments (media_fragment_pathes, P, env)
    # end def get_Media_R

    def render (self, context) :
        if "template" not in context :
            context.update (template = self)
        return self.template.render (context)
    # end def render

    @classmethod
    def _eval_fragments (cls, fragments, P, env = None) :
        from _CHJ import Parameters
        return Parameters.Scope (P, env).Eval (* fragments)
    # end def _eval_fragments

    def _load_media (self) :
        f_path = self.media_fragment_name
        if f_path is None :
            f_path = "%s.media" % (self.path.split ("::", 1) [-1], )
        try :
            source, path, _ = self.env.loader.get_source (self.env, f_path)
        except TemplateNotFound :
            pass
        else :
            self._media_fragment = self._coding_pat.sub ("", source, 1)
            self._media_path     = path
    # end def _load_media

    def _load_template (self) :
        try :
            source, path, _ = self.env.loader.get_source (self.env, self.path)
        except TemplateNotFound :
            pass
        else :
            self._t_path   = path
            self._t_source = source
    # end def _load_template

    def __eq__ (self, rhs) :
        return self.name == getattr (rhs, "name", rhs)
    # end def __eq__

    def __hash__ (self) :
        return hash (self.name)
    # end def __hash__

    def __lt__ (self, rhs) :
        sk = TFL.Sorted_By ()
        return sk (self.name) < sk (getattr (rhs, "name", rhs))
    # end def __lt__

    def __repr__ (self) :
        return "<JNJ.Template_E %s [%s]>" % (self.name, self.path)
    # end def __repr__

    def __str__ (self) :
        return "%s [%s]" % (self.name, self.path)
    # end def __str__

# end class Template_E

error_templates = \
    [ Template (400, "html/error_400.jnj")
    , Template (401, "html/error_401.jnj")
    , Template (403, "html/error_403.jnj")
    , Template (404, "html/error_404.jnj")
    , Template (405, "html/error_405.jnj")
    , Template (408, "html/error_408.jnj")
    , Template (410, "html/error_410.jnj")
    , Template (500, "html/error_500.jnj")
    , Template (503, "html/error_503.jnj")
    ]

Template ("account_change_email",         "html/change_email.jnj")
Template ("account_change_email_info",    "email/change_email_info.jnj")
Template ("account_change_password",      "html/change_password.jnj")
Template ("account_change_password_info", "email/change_password_info.jnj")
Template ("account_make_cert",            "html/make_client_cert.jnj")
Template ("account_register",             "html/register.jnj")
Template ("account_reset_password",       "html/reset_password.jnj")
Template ("account_reset_password_email", "email/reset_password.jnj")
Template ("account_verify_new_email",     "email/verify_new_email.jnj")
Template ("account_verify_email",         "email/verify_new_email.jnj")

Template ("mf3",           "html/mf3.m.jnj")
Template ("mf3_input",     "html/MF3/input.m.jnj")
Template ("mf3_h_cols",    "html/MF3/h_cols.m.jnj", parent_name = "mf3")

Template ("ETR_table",                    "html/ETR/table.m.jnj")

Template ("calendar",                     "html/calendar.jnj")
Template ("calendar_qx",                  "html/cal/wr.jnj")
Template ("calendar_day",                 "html/cal_day.jnj")
Template ("calendar_day_qx",              "html/cal/day.jnj")
Template ("calendar_week",                "html/cal_week.jnj")
Template ("console",                      "html/console.jnj")
Template ("default",                      "html/error.jnj")
Template ("empty_dir",                    "html/empty_dir.jnj")
Template ("error_email",                  "email/error.jnj")
Template ("e_type",                       "html/e_type.m.jnj")
Template ("e_type_admin",                 "html/e_type_admin.jnj")
Template ("e_type_display",               "html/e_type_display.jnj")
Template ("e_type_doc",                   "html/e_type_doc.jnj")
Template ("e_type_doc_bare",              "html/e_type_doc_bare.jnj")
Template ("e_type_mf3",                   "html/e_type_mf3.jnj")
Template ("e_type_selector",              "html/e_type_selector.m.jnj")
Template ("gallery",                      "html/gallery.jnj")
Template ("login",                        "html/login.jnj")
Template ("photo",                        "html/photo.jnj")
Template ("pns_e_type_doc",               "html/pns_e_type_doc.jnj")
Template ("site_admin",                   "html/site_admin.jnj")
Template ("video",                        "html/video.jnj")

class Templateer (TFL.Meta.Object) :
    """Encapsulate Jinja template handling"""

    Context = dict

    error_template_names = list (t.name for t in error_templates)

    def __init__ (self, * args, ** kw) :
        self.GTW = GTW = JNJ.GTW (self)
        self.env = env = JNJ.Environment.HTML (* args, GTW = GTW, ** kw)
        self.Template_Type = T = Template_E.New \
            ( "x"
            , By_Path      = {}
            , Map          = {}
            , Media_Map    = {}
            , css_href_map = {}
            , etag         = None
            )
        self.Template_Map  = T.Map
        for t in sorted (pyk.itervalues (Template.Map), key = TFL.Getter.id) :
            T.copy (env, t)
    # end def __init__

    @property
    def etag (self) :
        return self.Template_Type.etag
    # end def etag

    def call_macro (self, macro_name, * _args, ** _kw) :
        return self.GTW.call_macro (macro_name, * _args, ** _kw)
    # end def call_macro

    def get_template (self, name, injected = ()) :
        T      = self.Template_Type
        result = T (self.env, name)
        if injected :
            name_i = "%s|%s" % \
                ( result.name
                , "|".join (pyk.decoded (t.name) for t in sorted (injected))
                )
            if name_i in T.Map :
                result = T.Map [name_i]
            else :
                path   = result.path ### avoid lookup `T.By_Path [result.path]`
                args   = result.args [2:]
                result = T (self.env, name_i, None, * args, injected = injected)
                result.path = path
        return result
    # end def get_template

    def render (self, template_or_name, context) :
        template = template_or_name
        if isinstance (template_or_name, pyk.string_types) :
            template = self.get_template (template_or_name)
        return template.render (context)
    # end def render

    def render_string (self, template_string, context) :
        return self.env.from_string (template_string).render (context)
    # end def render_string

# end class Templateer

if __name__ != "__main__" :
    JNJ._Export ("*")
### __END__ JNJ.Templateer



# -*- coding: utf-8 -*-
# Copyright (C) 2015 Mag. Christian Tanzer All rights reserved
# Glasauergasse 32, A--1130 Wien, Austria. tanzer@swing.co.at
# #*** <License> ************************************************************#
# This module is part of the package JNJ.
#
# This module is licensed under the terms of the BSD 3-Clause License
# <http://www.c-tanzer.at/license/bsd_3c.html>.
# #*** </License> ***********************************************************#
#
#++
# Name
#    JNJ.SRM
#
# Purpose
#    Template declarations for SRM (sailing race management)
#
# Revision Dates
#    11-Feb-2015 (CT) Creation (factored from JNJ.Templateer)
#    ««revision-date»»···
#--

"""
Declare templates related to SRM (sailing race management).
"""

from   __future__ import division, print_function
from   __future__ import absolute_import, unicode_literals

from   _JNJ               import JNJ
from   _JNJ.Templateer    import Template

Template \
    ( "ETR_table_regatta",                "html/ETR/regatta.m.jnj"
    , parent_name = "ETR_table"
    )
Template \
    ( "ETR_table_regatta_register",       "html/ETR/regatta_register.m.jnj"
    , parent_name = "ETR_table_regatta"
    )
Template \
    ( "ETR_table_regatta_result",         "html/ETR/regatta_result.m.jnj"
    , parent_name = "ETR_table_regatta"
    )

Template ("regatta_calendar",             "html/regatta_calendar.jnj")
Template ("regatta_page",                 "html/regatta_page.jnj")
Template ("regatta_page_r",               "html/regatta_page_r.jnj")
Template ("regatta_ranking",              "html/regatta_ranking.jnj")
Template ("regatta_register_email",       "email/regatta_register.jnj")

if __name__ != "__main__" :
    JNJ._Export_Module ()
### __END__ JNJ.SRM



# -*- coding: utf-8 -*-
# Copyright (C) 2011-2016 Mag. Christian Tanzer All rights reserved
# Glasauergasse 32, A--1130 Wien, Austria. tanzer@swing.co.at
# ****************************************************************************
# This module is part of the package JNJ.
#
# This module is licensed under the terms of the BSD 3-Clause License
# <http://www.c-tanzer.at/license/bsd_3c.html>.
# ****************************************************************************
#
#++
# Name
#    JNJ.Media_Defaults
#
# Purpose
#    Provide defaults for media fragments in html/*.media
#
# Revision Dates
#     1-Jan-2011 (CT) Creation
#     2-Jan-2011 (CT) Creation continued
#     4-Jan-2011 (CT) Creation continued..
#    14-Jan-2011 (CT) Use `GTW.Parameters.Definition` to allow lazy
#                     references with `P` and `P_dict`
#    22-Mar-2011 (CT) `afs`
#    22-Mar-2011 (CT) `afs` continued
#    29-Mar-2011 (CT) `afs` continued..
#    30-Mar-2011 (CT) `afs` continued...
#    19-May-2011 (CT) `afs` continued....
#    16-Oct-2011 (MG) `Debugger` added
#    24-Oct-2011 (CT) `color_spec_heading_rev` added
#    24-Oct-2011 (CT) `tablesorter` added
#    24-Nov-2011 (CT) Add `color_spec_selected_rev`
#    23-Feb-2012 (CT) Add `color_status_{bad,good,missing}`, `afs.status_size`
#    23-Feb-2012 (CT) Add `color_bg_bad` and `color_bg_missing`
#     1-Mar-2012 (CT) Change `pg_main_max_width` from `Em (50)` to `Em (45)`
#     8-Mar-2012 (CT) Add `color_bg_menu`
#    10-Aug-2012 (CT) Add `color_spec_sub_heading`, `color_spec_term`
#     7-Mar-2013 (CT) Add `border_added` and `border_deleted`
#     3-Apr-2013 (CT) Add `Rule`, `Rule.clearfix`
#     8-Apr-2013 (CT) Add lots of rules to `Rule`, e.g., `disabled`, `focus`...
#     7-Jan-2014 (CT) Add some more muted colors
#    21-Jan-2014 (CT) Add `breakpoint`
#    20-Feb-2014 (CT) Add `Rule.pg_nav_show`, `.pg_nav_show_a`
#    20-Feb-2014 (CT) Add `breakpoint.broad` and `.very_narrow`
#     8-Apr-2014 (CT) Improve `clearfix`
#    12-Apr-2014 (CT) Use `Border.P`, not `P_Border`
#    15-May-2014 (CT) Add `color_lightest_grey`
#    15-May-2014 (CT) Add `breakpoint.quite_narrow`
#     9-Jul-2014 (CT) Add `Rule.rotate_45_left`
#    26-Aug-2014 (CT) Add `pure` parameters
#     3-Sep-2014 (CT) Add `Rule.hidden`
#     3-Dec-2014 (CT) Add `color.alphabet_max_contrast_colors` and
#                     `color.kellys_max_contrast_colors`
#    16-Jan-2015 (CT) Change `nav_col.width` and `col_padding` to unit `Rem`
#    23-Jan-2015 (CT) Add `color_jnd_grey`, reduce contrast of `color_spec_row1`
#    23-Jan-2015 (CT) Factor `line_height*` parameters
#    15-Feb-2015 (CT) Add `menu_icon`
#    24-Mar-2015 (CT) Add `Rule.input_focus`
#     8-Apr-2015 (CT) Change `font_family_print` to "serif" to void font
#                     substitution by the printer
#     2-Dec-2015 (CT) Fix `Rule.visited`
#     2-Dec-2015 (CT) Change `nav_col.color_spec_link_current.background_color`
#    31-Dec-2015 (CT) Change `pg_nav_show` to allow embedded rel-nav buttons
#    11-Oct-2016 (CT) Import from `CHJ`, not `GTW`
#    ««revision-date»»···
#--

from _CHJ._CSS.import_CSS import *
from _CHJ.Parameters      import \
    ( Definition, P, P_dict, Rule_Definition
    , Rule, Rule_Attr, Rule_Child, Rule_Class, Rule_Pseudo, Rule_Sibling
    )

Color.formatter = RGB_X

class Media_Defaults (Definition) :
    """Provide defaults for CSS fragments in html/*.css"""

    class color (Definition) :

        ### color alphabet
        ### http://eleanormaclure.files.wordpress.com/2011/03/colour-coding.pdf
        ###
        ### The RGB color values were taken with gcolor2's color picker from
        ### colour-coding.pdf
        abc_a                  = RGB_X     ("#C5A1CA")
        abc_b                  = RGB_X     ("#486EB5")
        abc_c                  = RGB_X     ("#82411D")
        abc_d                  = RGB_X     ("#401A57")
        abc_e                  = RGB_X     ("#1F1E1E")
        abc_f                  = RGB_X     ("#335A36")
        abc_g                  = RGB_X     ("#78C259")
        abc_h                  = RGB_X     ("#EDC99A")
        abc_i                  = RGB_X     ("#7F8080")
        abc_j                  = RGB_X     ("#BEDEAE")
        abc_k                  = RGB_X     ("#877A2F")
        abc_l                  = RGB_X     ("#A5C43A")
        abc_m                  = RGB_X     ("#9B247F")
        abc_n                  = RGB_X     ("#253777")
        abc_o                  = RGB_X     ("#E1A131")
        abc_p                  = RGB_X     ("#E6A4B5")
        abc_q                  = RGB_X     ("#50662E")
        abc_r                  = RGB_X     ("#CF2128")
        abc_s                  = RGB_X     ("#A4DBDF")
        abc_t                  = RGB_X     ("#56968C")
        abc_u                  = RGB_X     ("#E0E77B")
        abc_v                  = RGB_X     ("#584EA0")
        abc_w                  = RGB_X     ("#7D1416")
        abc_x                  = RGB_X     ("#F3F190")
        abc_y                  = RGB_X     ("#ECDA43")
        abc_z                  = RGB_X     ("#D55428")

        ### muted colors
        m_aqua                 = RGB_X     ("#7FDBFF")
        m_black                = RGB_X     ("#111111")
        m_blue                 = RGB_X     ("#0088DD")
        m_grey                 = RGB_X     ("#AAAAAA")
        m_fuchsia              = RGB_X     ("#F012BE")
        m_green                = RGB_X     ("#00AA00")
        m_lime                 = RGB_X     ("#01FF70")
        m_maroon               = RGB_X     ("#85144B")
        m_navy                 = RGB_X     ("#001F3F")
        m_red                  = RGB_X     ("#CC3333")
        m_olive                = RGB_X     ("#3D9970")
        m_orange               = RGB_X     ("#FFA022")
        m_pink                 = RGB_X     ("#DD4499")
        m_purple               = RGB_X     ("#AA33BB")
        m_silver               = RGB_X     ("#DDDDDD")
        m_teal                 = RGB_X     ("#33CCCC")
        m_white                = RGB_X     ("#EEEEEE")
        m_yellow               = RGB_X     ("#FFF00F")

        ### pure colors
        p_black                = SVG_Color ("black")
        p_blue                 = SVG_Color ("blue")
        p_cyan                 = SVG_Color ("cyan")
        p_gray = p_grey        = SVG_Color ("gray")
        p_green                = SVG_Color ("green")
        p_lime                 = SVG_Color ("lime")
        p_magenta              = SVG_Color ("magenta")
        p_maroon               = SVG_Color ("maroon")
        p_navy                 = SVG_Color ("navy")
        p_olive                = SVG_Color ("olive")
        p_purple               = SVG_Color ("purple")
        p_red                  = SVG_Color ("red")
        p_teal                 = SVG_Color ("teal")
        p_white                = SVG_Color ("white")
        p_yellow               = SVG_Color ("yellow")

        ### Kelly's 22 colors of maximum contrast
        ### http://www.iscc.org/pdf/PC54_1724_001.pdf
        ### http://burgess-studio.co.uk/colour/
        ### http://eleanormaclure.files.wordpress.com/2011/03/colour-coding.pdf
        ###
        ### The order of colors in Kelly's list was planned so that there
        ### would be maximum contrast between colors in a set if the required
        ### number of colors were always selected in order from the top. So a
        ### set of five colors should be white, black, yellow, purple, and
        ### orange. And if seven colors were required, light blue and red
        ### should be added. Kelly took care of the needs of people with
        ### defective color vision. The first nine colors would be maximally
        ### different for such people as well as for people with normal
        ### vision. These nine colors are also readily distinguishable by
        ### color name.
        ###
        ### The RGB color values were taken with gcolor2's color picker from
        ### colour-coding.pdf
        k_white                = RGB_X     ("#FFFFFF") #  1
        k_black                = RGB_X     ("#1F1E1E") #  2
        k_yellow               = RGB_X     ("#EBCD3F") #  3
        k_purple               = RGB_X     ("#6F308B") #  4
        k_orange               = RGB_X     ("#DB6A28") #  5
        k_light_blue           = RGB_X     ("#98CEE6") #  6
        k_red                  = RGB_X     ("#B91F36") #  7
        k_buff                 = RGB_X     ("#C1BC82") #  8
        k_gray = k_grey        = RGB_X     ("#7F8080") #  9 ———————————
        k_green                = RGB_X     ("#62A647") # 10
        k_purplish_pink        = RGB_X     ("#D386B1") # 11
        k_blue                 = RGB_X     ("#4578B4") # 12
        k_yellowish_pink       = RGB_X     ("#DD8565") # 13
        k_violet               = RGB_X     ("#493896") # 14
        k_orange_yellow        = RGB_X     ("#E1A131") # 15
        k_purplish_red         = RGB_X     ("#91278B") # 16
        k_greenish_yellow      = RGB_X     ("#E9E857") # 17
        k_reddish_brown        = RGB_X     ("#7D1716") # 18
        k_yellow_green         = RGB_X     ("#93AD3C") # 19
        k_yellowish_brown      = RGB_X     ("#6E3515") # 20
        k_reddish_orange       = RGB_X     ("#D12D27") # 21
        k_olive_green          = RGB_X     ("#2C3617") # 22

        kellys_max_contrast_colors = \
          [ k_white
          , k_black
          , k_yellow
          , k_purple
          , k_orange
          , k_light_blue
          , k_red
          , k_buff
          , k_gray
          , k_green
          , k_purplish_pink
          , k_blue
          , k_yellowish_pink
          , k_violet
          , k_orange_yellow
          , k_purplish_red
          , k_greenish_yellow
          , k_reddish_brown
          , k_yellow_green
          , k_yellowish_brown
          , k_reddish_orange
          , k_olive_green
          ]

        ### The order in the following list tries to maximize contrast,
        ### analogously to Kelly's list
        alphabet_max_contrast_colors = \
          [ abc_e
          , abc_y
          , abc_d
          , abc_z
          , abc_s
          , abc_r
          , abc_x
          , abc_i
          , abc_g
          , abc_p
          , abc_b
          , abc_h
          , abc_v
          , abc_o
          , abc_m
          , abc_u
          , abc_w
          , abc_l
          , abc_c
          , abc_a
          , abc_q
          , abc_j
          , abc_n
          , abc_k
          , abc_f
          , abc_t
          ]

    # end class color

    color_bg_bad           = RGB_X     ("#FFEEEE")
    color_bg_menu          = RGB_X     ("#DDEEFF")
    color_bg_missing       = RGB_X     ("#FFFFBB")
    color_desc             = RGB_X     ("#666666")
    color_focus            = SVG_Color ("yellow")
    color_heading          = RGB_X     ("#34444D")
    color_heading_closed   = RGB_X     ("#56666E")
    color_heading_sub      = RGB_X     ("#78888F")
    color_jnd_grey         = RGB_X     ("#F8F8F8")
    color_lightest_grey    = RGB_X     ("#F6F6F6")
    color_lighter_grey     = RGB_X     ("#EDEDED")
    color_light_grey       = RGB_X     ("#DEDEDE")
    color_border_grey      = RGB_X     ("#CCCCCC")
    color_medium_grey      = RGB_X     ("#BEBEBE")
    color_half_grey        = RGB_X     ("#888888")
    color_dark_grey        = RGB_X     ("#444444")
    color_darker_grey      = RGB_X     ("#222222")
    color_selected         = RGB_X     ("#FF6633")
    color_status_bad       = RGB_X     ("#FF6666")
    color_status_good      = RGB_X     ("#AAEEAA")
    color_status_missing   = RGB_X     ("#FFDD00")
    color_target           = RGB_X     ("#FF6633")

    css_arrow_color        = SVG_Color ("red")
    css_arrow_width        = Em (1./2)

    block_margin_bottom    = Em (1./2)

    border_added           = "1px solid "  + P.color.m_red
    border_button          = "2px outset " + P.color_medium_grey
    border_deleted         = "1px solid "  + P.color.m_blue
    border_simple          = "1px solid "  + P.color_medium_grey
    border_selected        = "2px solid "  + P.color_selected

    button_spec            = P_dict \
        ( border           = P.border_button
        , cursor           = "pointer"
        , margin           = TRBL0 (t = Em (1./4), b = Em (3./4))
        , padding          = TRBL  (Em (0.5), Em (1.5))
        , ** Border (radius = Px (10))
        )

    col_padding            = Rem (0.3125) ### Px (5)

    background_color       = P.color.p_white
    link_color             = RGB_X     ("#0000EE")
    no_link_color          = RGB_X     ("#333333")
    text_color             = RGB_X     ("#000033")
    visited_color          = RGB_X     ("#551A8B")

    class afs (Definition) :

        block_margin_bottom    = Em (0.1)

        border_spec_input      = Border.P \
            ( color            = P.R.color_dark_grey
            , style            = "solid"
            , width            = Px (1)
            )
        border_spec_readonly   = Border.P \
            ( color            = P.R.color_medium_grey
            , style            = "solid"
            , width            = Px (2)
            )
        border_spec_section    = Border.P \
            ( color            = P.R.color_darker_grey
            , style            = "solid"
            , width            = TRBL0 (l = Px (2), default = Px (1))
            )

        color_spec_desc        = P_dict \
            ( background_color = P.R.color_desc
            , color            = P.R.background_color
            )
        color_spec_heading     = P_dict \
            ( background_color = P.R.color_heading
            , color            = P.R.background_color
            )
        color_spec_heading_closed = P_dict \
            ( background_color = P.R.color_heading_closed
            , color            = P.R.background_color
            )
        color_spec_heading_sub = P_dict \
            ( background_color = P.R.color_heading_sub
            , color            = P.R.background_color
            )
        color_spec_label       = P_dict \
            ( background_color = "inherit"
            , color            = P.R.text_color
            )
        color_spec_optional    = P.R.color_spec_normal
        color_spec_necessary   = P_dict \
            ( background_color = P.R.color_light_grey
            , color            = P.R.text_color
            )
        color_spec_readonly    = P_dict \
            ( background_color = P.R.color_light_grey
            , color            = P.R.color_dark_grey
            )
        color_spec_required    = P_dict \
            ( background_color = P.R.color_medium_grey
            , color            = P.R.text_color
            )

        header_padding         = TRBL (Em (0.2), Em (0.2), Em (0.3), Em (0.5))
        status_size            = Px (12)

    # end class afs

    class breakpoint (Definition) :
        """Breakpoints for responsive rules"""

        very_narrow            = P_dict \
            ( max_width        = Px (420)
            )

        quite_narrow           = P_dict \
            ( max_width        = Px (480)
            )

        narrow                 = P_dict \
            ( max_width        = Px (680)
            )

        small_device           = P_dict \
            ( max_device_width = Px (767)
            )

        broad                  = P_dict \
            ( min_width        = Px (1280)
            )

        wide                   = P_dict \
            ( min_width        = Px (1600)
            )

    # end class breakpoint

    class menu_icon (Definition) :

        color                  = P.R.color_selected
        line_width             = Rem (0.1875)
        margin_ab              = Em (0.4375)
        margin_m               = Em (0.125)
        width                  = Em  (1.25)

    # end class menu_icon

    class nav_col (Definition) :

        background_color       = RGB_X     ("#EFF3FE")

        button_spec            = P_dict \
            ( border           = P.R.border_button
            , color            = P.R.link_color
            , margin           = 0
            , text_align       = "center"
            )

        color_spec_heading     = P_dict \
            ( background_color = P.R.color_heading
            , color            = P.background_color
            )

        color_spec_heading_rev = P_dict \
            ( background_color = P.background_color
            , color            = P.R.color_heading
            )

        color_spec_label       = P_dict \
            ( background_color = P.background_color
            , color            = P.R.text_color
            )

        color_spec_link        = P_dict \
            ( background_color = P.background_color
            , color            = P.R.link_color
            )
        color_spec_link_current    = P_dict \
            ( background_color = P.R.visited_color
            , color            = P.background_color
            )

        color_spec_no_link     = P_dict \
            ( background_color = P.background_color
            , color            = P.R.no_link_color
            )

        color_spec_section_current = P_dict \
            ( background_color = P.background_color
            , color            = P.R.color_heading
            )

        color_spec_visited     = P_dict \
            ( color_spec_link
            , color            = P.R.visited_color
            )

        color_spec_web_link_hover = P_dict \
            ( color_spec_link
            , background_color = RGB_8 (255, 153, 0)
            )

        li_left                = Em (0.75)
        line_height_larger     = 1.50
        line_height_normal     = 1.35
        mark_color_link        = P.R.css_arrow_color
        mark_color_section     = P.R.color_heading
        mark_width             = Em (0.40)

        vert_padding           = Rem (0.2)
        width                  = Rem (11.875) ### Px (190)

        full_width             = P.width      + 2 * P.R.col_padding
        right                  = P.full_width + 2 * P.R.col_padding

    ### end class nav_col

    class cal (Definition) :

        date_bg                = P.R.color_medium_grey
        date_padding           = TRBL0 (r = Em (1./4), b = Em (1./10))
        event_bg               = RGB_X ("#FFF8AF")
        font_size              = Em    (0.7)
        line_height            = 1.5
        holiday_bg             = RGB_X ("#CCFFFF")
        month_color            = RGB_X ("#777777")
        weekend_color          = P.R.color.m_blue
        week_bg                = P.R.color_heading
        week_color             = RGB_8 (255, 153, 0)
        week_height            = Em    (8)

    # end class cal

    class pure (Definition) :
        """Parameters of `pure` css as of version v0.4.2."""

        input_focus_border_color = RGB_X ("#129FEA")

        label_width              = Em (10.0)
        label_margin_right       = Em (1.0)

        aside_indent             = label_width + label_margin_right

    # end class pure

    class tablesorter (Definition) :

        color_marker           = P.R.background_color
        margin_top             = Px (8)
        opacity                = 0.75
        width                  = Px (5)

    # end class tablesorter

    class Rule (Rule_Definition) :

        clearfix           = Rule_Pseudo \
            ( "after"
            , clear        = "both"
            , content      = "' '"
            , display      = "table"
            # http://nicolasgallagher.com/micro-clearfix-hack/
            )

        disabled           = Rule_Class \
            ( "disabled"
            , opacity      = 0.5
            )

        focus              = Rule_Pseudo \
            ( "focus"
            ,  P.R.color_spec_selected
            )

        focus_outline      = Rule_Pseudo \
            ( "focus"
            ,  P.R.color_spec_selected
            , outline      = P.R.outline_focus
            )

        hidden             = Rule \
            ( display      = "none"
            , visibility   = "hidden"
            )

        hover              = Rule_Pseudo \
            ( "hover"
            ,  P.R.color_spec_selected
            )

        hover_rev          = Rule_Pseudo \
            ( "hover"
            ,  P.R.color_spec_selected_rev
            )

        input_focus        = Rule_Pseudo \
            ( "focus"
            , background_color = "inherit"
            , border_color     = P.R.pure.input_focus_border_color
            , color            = RGB_X ("#0078E7")
            , outline          = P.R.pure.input_focus_border_color
            )

        link               = Rule_Pseudo \
            ( "link"
            , color = P.R.link_color
            )

        pg_nav_show        = Rule \
            ( ".pg_nav_show"
            , background_color = P.R.nav_col.background_color
            , display          = "block"
            , font_size        = Percent (200)
            , max_width        = Percent (100)
            , overflow         = "hidden"
            , visibility       = "visible"
            , width            = Vw (100)
            , children         = [P.R.Rule.pg_nav_show_a]
            )

        pg_nav_show_a      = Rule_Child \
            ( "a"
            , color            = P.R.color_selected
            , font_weight      = "bold"
            , display          = "inline-block"
            , padding          = TRBL (Em (0.25), Em (0.50))
            , children         = [P.R.Rule.hover]
            , ** Border (radius = Px (10))
            )

        rotate_45_left     = Rule \
            ( Transform ("rotate(-45deg)")
            , display          = "inline-block"
            )

        row_even           = Rule_Pseudo \
            ( "nth-child(2n)"
            , P.R.color_spec_row2
            )

        row_odd            = Rule_Pseudo \
            ( "nth-child(2n+1)"
            , P.R.color_spec_row1
            )

        target             = Rule_Pseudo \
            ( "target"
            ,  P.R.color_spec_selected
            )

        target_outline     = Rule_Pseudo \
            ( "target"
            , P.R.color_spec_selected
            , outline      = P.R.outline_focus
            )

        visited            = Rule_Pseudo \
            ( "visited"
            , color        = P.R.visited_color
            )

    # end class Rule

    color_spec_error       = P_dict \
        ( background_color = P.background_color
        , color            = P.color_status_bad
        )
    color_spec_gallery_heading = P_dict \
        ( background_color = RGB_P     (50, 75, 100)
        , color            = P.nav_col.background_color
        )
    color_spec_heading     = P_dict \
        ( background_color = P.background_color
        , color            = P.color_selected
        )
    color_spec_normal      = P_dict \
        ( background_color = P.background_color
        , color            = P.text_color
        )
    color_spec_pg_head     = P_dict \
        ( background_color = P.background_color
        , color            = RGB_X     ("#0200DE")
        )
    color_spec_row1        = color_spec_meta = P_dict \
        ( background_color = "transparent"
        , color            = P.text_color
        )
    color_spec_row2        = color_spec_message = P_dict \
        ( background_color = P.color_lightest_grey
        , color            = P.text_color
        )
    color_spec_selected    = P_dict \
        ( background_color = SVG_Color ("yellow")
        , color            = SVG_Color ("red")
        )
    color_spec_selected_rev= P_dict \
        ( background_color = SVG_Color ("red")
        , color            = SVG_Color ("yellow")
        )
    color_spec_strong      = P_dict \
        ( background_color = P.background_color
        , color            = SVG_Color ("blue")
        )
    color_spec_sub_heading = P_dict \
        ( background_color = P.background_color
        , color            = P.color_half_grey
        )
    color_spec_term        = P_dict \
        ( background_color = RGB_X ("#E6E6E6")
        , color            = P.color_dark_grey
        )

    del_spec               = P_dict \
        ( text_decoration  = "line-through"
        # XXX ???
        )

    font_family_normal     = """"Lucida Grande", verdana, sans-serif"""
    font_family_pre        = \
        """"Lucida Sans Typewriter", "Lucida Console", "Courier New", Courier, monospace"""
    font_family_print      = "serif"
        ### Don't use specific fonts for `print` because font substitution done
        ### by a printer can look terribly ugly
    font_spec_normal       = P_dict \
        ( font_family      = P.font_family_normal
        , font_style       = "normal"
        , font_weight      = "normal"
        , line_height      = P.line_height_normal
        )

    font_spec_print        = P_dict \
        ( font_spec_normal
        , font_family      = P.font_family_print
        )

    font_spec_em           = P_dict \
        ( font_spec_normal
        , font_weight      = "bold"
        )
    font_spec_pre          = P_dict \
        ( font_spec_normal
        , font_family      = P.font_family_pre
        )

    grid_table_border      = "3px ridge gray"

    h1_font_size           = Percent (125)
    h1_font_weight         = "bold"

    hbox_spec              = P_dict \
        ( display          = "block"
        , overflow         = "hidden"
        )

    hr_spec                = P_dict \
        ( border           = 0
        , border_top       = "1px solid #CCC"
        , display          = "block"
        , height           = Px (1)
        , margin           = TRBL (Em (1), 0)
        , padding          = 0
        )

    input_margin           = TRBL (Em (0.1), 0)
    input_padding          = Em (0.2)

    ins_spec               = P_dict \
        ( text_decoration  = "none"
        # XXX ???
        )

    line_height_heading    = 2.00
    line_height_larger     = 1.875
    line_height_normal     = 1.44
    line_height_input      = 1.143

    outline_focus          = "2px solid "   + P.color_focus
    outline_target         = "2px dotted "  + P.color_target

    pg_head_height         = Px (100)
    pg_main_max_width      = Em (45)
    pg_main_min_width      = Em (15)

    thumbnail_size         = Px (155)
    thumbnail_selected_color = P.color_selected

    class Debugger (Definition) :

        background_color           = P.R.background_color
        console_border_color       = P.R.color_border_grey
        console_background_color   = P.R.color_lightest_grey
        console_text_color         = P.R.color.m_black

        form_text_color            = P.R.color_dark_grey

        traceback_background_color = P.R.background_color
        traceback_text_color       = P.R.background_color

    # end class Debugger

# end class Media_Defaults

### __END__ JNJ.Media_Defaults



# -*- coding: utf-8 -*-
# Copyright (C) 2009-2016 Mag. Christian Tanzer All rights reserved
# Glasauergasse 32, A--1130 Wien, Austria. tanzer@swing.co.at
# ****************************************************************************
#
# This module is licensed under the terms of the BSD 3-Clause License
# <http://www.c-tanzer.at/license/bsd_3c.html>.
# ****************************************************************************
#
#++
# Name
#    JNJ.Environment
#
# Purpose
#    Provide wrapper around jinja2.Environment
#
# Revision Dates
#    29-Dec-2009 (CT) Creation
#    13-Jan-2010 (CT) `GTW` converted from module to class instance
#    21-Jan-2010 (MG) `I18N` support added
#    21-Jan-2010 (CT) Bugs fixed
#    22-Jan-2010 (CT) PrefixLoader for `STD::` added to allow access
#                     to shadowed templates of JNJ itself
#    23-Feb-2010 (MG) `Default_Extensions` factored
#     3-Jan-2011 (CT) `CSS_Parameters` added
#    16-Mar-2011 (CT) Optional argument `GTW` added to `HTML`
#    29-Mar-2012 (CT) Rename `CSS_Parameters` to `Media_Parameters`
#    18-Nov-2013 (CT) Change default `encoding` to `utf-8`
#     9-Jul-2014 (CT) Add `prefixes` argument to `HTML`
#    29-Oct-2015 (CT) Improve Python 3 compatibility
#    ««revision-date»»···
#--

from   __future__  import absolute_import
from   __future__  import division
from   __future__  import print_function
from   __future__  import unicode_literals

from   _JNJ               import JNJ
from   _TFL               import TFL

import _JNJ.Onion
import _JNJ.GTW

from   _TFL               import sos
from   _TFL.predicate     import uniq
from   _TFL.pyk           import pyk
import _TFL.I18N

from jinja2 import Environment, FileSystemLoader, ChoiceLoader, PrefixLoader

Default_Extensions = ["jinja2.ext.loopcontrols", "jinja2.ext.do", JNJ.Onion]

def HTML \
        ( version          = "html/5.jnj"
        , load_path        = ()
        , loader           = None
        , globals          = {}
        , encoding         = "utf-8"
        , i18n             = False
        , Media_Parameters = None
        , GTW              = None
        , prefixes         = {}
        , ** kw
        ) :
    jnj_loader = FileSystemLoader (sos.path.dirname (__file__), "utf-8")
    loaders    = []
    if loader :
        loaders.append (loader)
    if load_path :
        loaders.append (FileSystemLoader (load_path, encoding))
    loaders.append (jnj_loader)
    if prefixes :
        sk = lambda x : (- len (x [1]), x [1])
        for prefix, lp in sorted (pyk.iteritems (prefixes), key = sk) :
            loaders.append \
                ( PrefixLoader
                   ({prefix : FileSystemLoader (lp, encoding)}, delimiter='::')
                )
    loaders.append (PrefixLoader (dict (STD = jnj_loader), delimiter='::'))
    loader     = ChoiceLoader (loaders)
    extensions = (kw.pop ("extensions", []) + Default_Extensions)
    if i18n :
        extensions.append ("jinja2.ext.i18n")
    result = Environment  \
        ( extensions = uniq (extensions), loader = loader, ** kw)
    if GTW is None :
        GTW = JNJ.GTW (result)
    result.globals.update \
        ( globals
        , GTW          = GTW
        , html_version = version
        )
    result.Media_Parameters = Media_Parameters
    result.encoding         = encoding
    result.static_handler   = None
    if i18n :
        result.install_gettext_translations (TFL.I18N)
    return result
# end def HTML

__doc__ = """
`HTML` creates an environment suitable for generating HTML from jinja
templates. Per default, `HTML` sets `html_version` to a macro template
for HTML5.

    >>> env5 = HTML ()

Passing `version = "html/x.jnj"` to `HTML` sets up an environment with
`html_version` referring to a macro template for XHTML.

    >>> envx = HTML (version = "html/x.jnj")

By importing `html_version` as `x` and calling macros defined by `X`,
a single template can generate either HTML5 or XHTML output, depending
on how the environment was set up::

    >>> template = '''
    ... {%- import html_version as X -%}
    ... {%- call X.html (lang = "de") -%}
    ...   {%- call X.section (id = "foo", class = "bar") -%}
    ...     A sample section for testing the differences between
    ...     html5 and xhtml.
    ...   {%- endcall -%}
    ...
    ...   {%- set a = None %}
    ...   {%- set b = 42 %}
    ...   {%- set c = "cccc" %}
    ...   {{ GTW.firstof (a, b, c, d, "default first of") }}
    ...   {{ GTW.firstof (b, c, d, "default first of") }}
    ...   {{ GTW.firstof (c, d, "default first of") }}
    ...   {{ GTW.firstof (d, "default first of") }}
    ...   {{ GTW.firstof ("default first of") }}
    ...
    ...   {{ X.input.textarea (80, 5, value = "Wing is king", class = "fast") }}
    ...
    ...   {{ X.input.select ("Choice", ["red", "green"], class = "side") }}
    ...   {{ X.input.select_v ("CV", [(0, "red"), (1, "green")], 1, class = "side") }}
    ... {%- endcall -%}
    ... '''

    >>> t5 = env5.from_string (template)
    >>> print (t5.render ().replace ('<section id="foo" class="bar">', '<section class="bar" id="foo">'))
    <!DOCTYPE html>
    <html lang="de" class="no-js">
      <section class="bar" id="foo">
      A sample section for testing the differences between
        html5 and xhtml.
    </section>
      42
      42
      cccc
      default first of
      default first of
    <BLANKLINE>
      <textarea cols="80" rows="5" class="fast">Wing is king</textarea>
    <BLANKLINE>
    <BLANKLINE>
      <select class="side" name="Choice">
    <BLANKLINE>
          <option value="">---</option>
    <BLANKLINE>
            <option>red</option>
    <BLANKLINE>
            <option>green</option>
    <BLANKLINE>
        </select>
    <BLANKLINE>
      <select class="side" name="CV">
    <BLANKLINE>
    <BLANKLINE>
            <option value="0">red</option>
            <option value="1" selected="selected">green</option>
    <BLANKLINE>
            <option value="">---</option>
        </select>
    </html>

    >>> et = env5.get_template ("email/email.jnj")
    >>> print (et.render (email_from = "tanzer@swing.co.at", email_to = "martin@mangari.com", email_body = "Nih!", NAV = None))
    From:    tanzer@swing.co.at
    To:      martin@mangari.com
    Content-type: text/plain; charset=iso-8859-15
    <BLANKLINE>
    <BLANKLINE>
    Nih!

    >>> print (et.render (email_from = "tanzer@swing.co.at", email_to = "martin@mangari.com", email_body = "Nih!", NAV = None))
    From:    tanzer@swing.co.at
    To:      martin@mangari.com
    Content-type: text/plain; charset=iso-8859-15
    <BLANKLINE>
    <BLANKLINE>
    Nih!

"""

if __name__ != "__main__" :
    JNJ._Export_Module ()
### __END__ JNJ.Environment



# -*- coding: utf-8 -*-
# Copyright (C) 2009-2016 Mag. Christian Tanzer All rights reserved
# Glasauergasse 32, A--1130 Wien, Austria. tanzer@swing.co.at
# ****************************************************************************
#
# This module is licensed under the terms of the BSD 3-Clause License
# <http://www.c-tanzer.at/license/bsd_3c.html>.
# ****************************************************************************
#
#++
# Name
#    JNJ.GTW
#
# Purpose
#    Provide additional global functions for Jinja templates
#
# Revision Dates
#    29-Dec-2009 (CT) Creation
#    13-Jan-2010 (CT) Converted to class; `call_macro` added;
#                     `_T` and `_Tn` added to class `GTW`
#    25-Jan-2010 (MG) `_T` and `_Tn` need to be static methods
#    27-Jan-2010 (CT) `Getter`, `now`, and `sorted` added
#    17-Feb-2010 (CT) `email_uri`, `obfuscated`, `tel_uri`, and `uri` added
#    23-Feb-2010 (CT) `pjoin` added
#    24-Feb-2010 (CT) `log_stdout` added
#    10-Mar-2010 (CT) `zip` added
#     3-May-2010 (MG) `call_macro`: `widget_type` added
#     5-May-2010 (MG) `render_fofi_widget` added
#     5-May-2010 (MG) `default_render_mode` used
#     5-May-2010 (MG) `render_mode` added
#     6-May-2010 (MG) `render_fofi_widget` exception handling improoved
#     3-Aug-2010 (CT) Use `HTML.obfuscator` instead of home-grown code
#     3-Aug-2010 (CT) `obfuscated` removed
#    20-Sep-2010 (CT) `Sorted_By` added
#    22-Sep-2010 (CT) `eval_sorted_by` added
#     8-Oct-2010 (CT) `len` added
#    23-Nov-2010 (CT) `list` and `reversed` added
#    27-Nov-2010 (CT) `formatted` added
#    18-Mar-2011 (CT) `get_macro` changed to use `Template_E.get_macro`
#    22-Mar-2011 (CT) `dict` added
#    30-Nov-2011 (CT) Add `filtered_join`
#    30-Nov-2011 (CT) Add `dir` and `getattr`
#     1-Dec-2011 (CT) Add `styler`
#    18-Jan-2012 (CT) Add `attr_join`
#    27-Jan-2012 (CT) Change `email_uri` to allow tuple argument for `email`
#    22-Feb-2012 (CT) Add `vimeo_video` and `youtube_video`
#     4-May-2012 (CT) Change `email_uri` to allow email-tuple and `text` passed
#    16-Jul-2012 (MG) `log_stdout` enhanced
#     6-Aug-2012 (MG) Add `update_blackboard`
#     8-Aug-2012 (MG) Remove debug code
#     9-Aug-2012 (MG) Use `** kw` notation for `update_blackboard`
#    12-Feb-2014 (CT) Add `enumerate`
#    13-Feb-2014 (CT) Add `Dingbats` and `unichr`
#    14-Mar-2014 (CT) Add `any` and `all`
#    10-Apr-2014 (CT) Add `first`
#    14-Apr-2014 (CT) Add `ichain`
#    18-Apr-2014 (CT) Add `bool`
#     2-Dec-2014 (CT) Add `setattr`
#    21-Jan-2015 (CT) Add `filtered_dict`
#    23-Jan-2015 (CT) Add `html_char_ref`
#    10-Jun-2015 (CT) Add `uuid`
#    13-Nov-2015 (CT) Define `dir`, `setattr` as functions to avoid sphinx errors
#    16-May-2016 (CT) Add `xmlattr`
#    16-May-2016 (CT) Add guard for `Undefined` to `filtered_dict`
#    11-Oct-2016 (CT) Change `GTW.HTML` to `TFL.HTML`
#    ««revision-date»»···
#--

from   __future__               import print_function

from   _JNJ                     import JNJ
from   _TFL                     import TFL

from   _TFL.I18N                import _, _T, _Tn
from   _TFL.predicate           import filtered_join
from   _TFL.pyk                 import pyk
from   _TFL                     import sos

from   _TFL._Meta.Once_Property import Once_Property

import _TFL._Meta.Object
import _TFL.Accessor
import _TFL.Caller
import _TFL.HTML
import _TFL.Sorted_By

import itertools
import uuid

class GTW (TFL.Meta.Object) :
    """Provide additional global functions for Jinja templates."""

    from jinja2.runtime import Undefined

    def __init__ (self, env) :
        import _JNJ.Templateer ### used by `get_macro`
        self.env               = env
        self.render_mode_stack = []
    # end def __init__

    all  = staticmethod (all)
    any  = staticmethod (any)
    bool = staticmethod (bool)

    @Once_Property
    def Dingbats (self) :
        """Provide access to :mod:`~_TFL.Dingbats`."""
        from _TFL import Dingbats
        return Dingbats
    # end def Dingbats

    def attr_join (self, sep, objects, attr_name) :
        """Join the values of attribute `attr_name` of `objects` by `sep`."""
        def _gen (objects, attr_name) :
            for o in objects :
                a = getattr (o, attr_name)
                if a :
                    yield a
        return sep.join (_gen (objects, attr_name))
    # end def attr_join

    def call_macro (self, macro_name, * _args, ** _kw) :
        """Call macro named by `macro_name` passing `* _args, ** _kw`."""
        templ_name  = _kw.pop ("templ_name",  None)
        widget_type = _kw.pop ("widget_type", None)
        try :
            macro  = self.get_macro (macro_name, templ_name, widget_type)
        except ValueError :
            print (repr ((macro_name, templ_name, _args, _kw)))
            raise
        return macro (* _args, ** _kw)
    # end def call_macro

    dict = staticmethod (dict)

    def dir (self, x = None) :
        """Return directory of `x` or the locals of the caller."""
        if x is None :
            return sorted (TFL.Caller.locals ())
        else :
            return dir (x)
    # end def dir

    def email_uri (self, email, text = None, ** kw) :
        """Returns a mailto URI for `email`.

           http://tools.ietf.org/html/rfc3966
        """
        if isinstance (email, tuple) :
            email, email_text = email
        if text is None :
            text = email_text
        return self.uri (scheme = "mailto", uri = email, text = text, ** kw)
    # end def email_uri

    enumerate = staticmethod (enumerate)

    def eval_sorted_by (self, key = None) :
        """Returns a function that can be passed to `Sorted_By` to evaluate
           the `sorted_by` attribute of an object, If `key` is passed in, the
           `sorted_by` attribute of the attribute referred to by `key` will
           be evaluated.
        """
        if key is None :
            result = lambda obj : obj.sorted_by (obj)
        else :
            getter = getattr (TFL.Getter, key)
            def result (obj) :
                v = getter (obj)
                return v.sorted_by (v)
        return result
    # end def eval_sorted_by

    def filtered_dict (self, * ds, ** kw) :
        """Return a dict will all non-empty values of `ds` and `kw`."""
        Undef  = self.Undefined
        result = {}
        for d in ds :
            result.update (d)
        result.update (kw)
        return dict \
            (  (k, v) for k, v in pyk.iteritems (result)
            if v is not None and v != "" and not isinstance (v, Undef)
            )
    # end def filtered_dict

    filtered_join = staticmethod (filtered_join)
    first         = staticmethod (TFL.first)

    def firstof (self, * args) :
        """Return the first non-None and non-Undefined value of `args`.

           If `args` contains a single `tuple` or `list`, returns first
           non-None and non-Undefined element of that sequence.
        """
        if len (args) == 1 and isinstance (args [0], (tuple, list)) :
            args = args [0]
        for a in args :
            if not (a is None or isinstance (a, self.Undefined)) :
                return a
    # end def firstof

    def formatted (self, format, * args, ** kw) :
        """Return result of `%` applied to `format`and `args` or `kw`."""
        if args :
            assert not kw
            return format % args
        else :
            return format % kw
    # end def formatted

    def get_macro (self, macro_name, templ_name = None, widget_type = None) :
        """Return macro `macro_name` from template `templ_name`."""
        if widget_type :
            macro_name = getattr (macro_name, widget_type)
        if not isinstance (macro_name, pyk.string_types) :
            macro_name = str (macro_name)
        if templ_name is None :
            templ_name, macro_name = \
                (p.strip () for p in macro_name.split (",", 1))
        template = self.env.get_template (templ_name)
        if isinstance (template, JNJ.Template_E) :
            result = template.get_macro (macro_name)
        else :
            result = getattr (template.module, macro_name)
        return result
    # end def get_macro

    getattr    = staticmethod (getattr)
    Getter     = TFL.Getter

    def html_char_ref (self, arg) :
        """Return HTML character reference for `arg`."""
        if isinstance (arg, pyk.string_types) and len (arg) == 1 :
            arg = ord (arg)
        return "&#x%4.4X" % (arg, )
    # end def html_char_ref

    ichain     = staticmethod (itertools.chain)
    len        = staticmethod (len)
    list       = staticmethod (list)

    def log_stdout (self, * text) :
        """Write `text` to standard output."""
        print (* text)
        return ""
    # end def log_stdout

    def now (self, format = "%Y/%m/%d") :
        """Return the current date-time formatted by `format`."""
        from datetime import datetime
        result = datetime.now ()
        return result.strftime (format)
    # end def now

    pjoin      = staticmethod (sos.path.join)

    def render_fofi_widget (self, fofi, widget, * args, ** kw) :
        pushed          = False
        obj_render_mode = getattr (fofi, "render_mode", None)
        kw_render_mode  = kw.pop  ("render_mode",       None)
        if obj_render_mode :
            pushed      = True
            self.render_mode_stack.append (obj_render_mode)
        elif kw_render_mode :
            pushed      = True
            self.render_mode_stack.append (kw_render_mode)
        elif not self.render_mode_stack :
            pushed      = True
            self.render_mode_stack.append (fofi.default_render_mode)
        render_mode     = self.render_mode_stack [-1]
        try :
            try :
                mode_desc = fofi.render_mode_description [render_mode]
            except ValueError :
                raise ValueError \
                    ("%r does not support render mode %r" % (fofi, render_mode))
            result      = self.call_macro \
                (getattr (mode_desc, widget), * args, ** kw)
            return result
        finally :
            if pushed :
                self.render_mode_stack.pop ()
    # end def render_fofi_widget

    @Once_Property
    def render_mode (self) :
        return self.render_mode_stack and self.render_mode_stack [-1]
    # end def render_mode

    reversed   = staticmethod (reversed)

    def setattr (self, x, y, v) :
        """Set attribute `y` of object `x` to value `v`."""
        setattr (x, y, v)
    # end def setattr

    sorted     = staticmethod (sorted)
    Sorted_By  = TFL.Sorted_By
    styler     = staticmethod (TFL.HTML.Styler)

    def tel_uri (self, phone_number, text = None, ** kw) :
        """Returns a telephone URI for `phone_number`.

           http://tools.ietf.org/html/rfc3966
        """
        return self.uri (scheme = "tel", uri = phone_number, text = text, ** kw)
    # end def tel_uri

    def update_blackboard (self, ** kw) :
        """Add `kw` to blackboard."""
        self.blackboard.update (kw)
        return ""
    # end def update_blackboard

    unichr = staticmethod (pyk.unichr)

    def uri (self, scheme, uri, text = None, ** kw) :
        """Return HTML <a> element as specified by `scheme`, `uri`, `text`, and
           `kw`.

           If `kw` contains a true value for `obfuscate`,
           :func:`~_TFL.HTML.obfuscator` will be applied to the result.

           All other values in `kw` will be converted to attributes of the <a>
           element.
        """
        obfuscate = kw.pop ("obfuscate", False)
        if text is None :
            text = uri
        attrs = ['href="%s:%s"' % (scheme, uri)]
        for k, v in pyk.iteritems (kw) :
            attrs.append ('%s="%s"' % (k, v))
        attrs = " ".join (sorted (attrs))
        result = u"""<a %(attrs)s>%(text)s</a>""" % locals ()
        if obfuscate :
            result = TFL.HTML.obfuscator [scheme] (result)
        return result
    # end def uri

    def uuid (self) :
        """Return the hex-value of a newly created UUID (uuid1)."""
        return uuid.uuid1 ().hex
    # end def uuid

    vimeo_video   = staticmethod (TFL.HTML.vimeo_video)
    youtube_video = staticmethod (TFL.HTML.youtube_video)

    def xmlattr (self, * ds, ** kw) :
        """Convert (sorted) items of dict `d` to SGML/XML attribute string.

           This is similar to jinja's `xmlattr` filter but ensures
           deterministic output by sorting by attribute name.
        """
        from jinja2.utils import escape
        d      = self.filtered_dict (* ds, ** kw)
        result = " ".join \
            ( '%s="%s"' % (escape (k), escape (v))
            for k, v in sorted (pyk.iteritems (d), key = TFL.Getter [0])
            )
        return (" " + result) if result else ""
    # end def xmlattr

    zip           = staticmethod (zip)
    _T            = staticmethod (_T)
    _             = staticmethod (_)
    _Tn           = staticmethod (_Tn)

# end class GTW

if __name__ != "__main__" :
    JNJ._Export ("GTW")
### __END__ JNJ.GTW



# -*- coding: utf-8 -*-
# Copyright (C) 2009-2015 Mag. Christian Tanzer All rights reserved
# Glasauergasse 32, A--1130 Wien, Austria. tanzer@swing.co.at
# ****************************************************************************
#
# This module is licensed under the terms of the BSD 3-Clause License
# <http://www.c-tanzer.at/license/bsd_3c.html>.
# ****************************************************************************
#
#++
# Name
#    JNJ.Onion
#
# Purpose
#    Provide Jinja extension for a `onion` tag
#
# Revision Dates
#    29-Dec-2009 (CT) Creation
#    12-Oct-2014 (CT) Use `next` function, not method (Python-3 compatibility)
#    29-Oct-2015 (CT) Improve Python 3 compatibility
#    11-Dec-2015 (CT) Fix doc-string of `Onion`
#    ««revision-date»»···
#--

from   _JNJ               import JNJ

from   jinja2     import nodes
from   jinja2.ext import Extension

class Onion (Extension) :
    """Jinja extension providing a `onion` tag.

       >>> from _JNJ.Environment import HTML
       >>> from _TFL.portable_repr import portable_repr
       >>> env = HTML ()
       >>> template = '''
       ...   {%- onion foo %}
       ...     {% head %}
       ...       I am the onion then head
       ...     {% else %}
       ...       I am the onion else head
       ...     {% body %}
       ...       And this is the body which should be enclosed by the head/tail
       ...     {% tail %}
       ...       I am the onion then tail
       ...     {% else %}
       ...       I am the onion else tail
       ...   {% endonion -%}
       ... '''
       >>> t = env.from_string (template)
       >>> print (portable_repr (t.render (foo = True)))
       '\\n      I am the onion then head\\n    \\n      And this is the body which should be enclosed by the head/tail\\n    \\n      I am the onion then tail\\n    '
       >>> print (portable_repr (t.render (foo = False)))
       '\\n      I am the onion else head\\n    \\n      And this is the body which should be enclosed by the head/tail\\n    \\n      I am the onion else tail\\n  '

       The else tags inside the head/tail are optional:

       >>> template = '''
       ...   {%- onion foo %}
       ...     {% head %}
       ...       I am the onion then head
       ...     {% body %}
       ...       And this is the body which should be enclosed by the head/tail
       ...     {% tail %}
       ...       I am the onion then tail
       ...     {% else %}
       ...       I am the onion else tail
       ...   {% endonion -%}
       ... '''
       >>> t = env.from_string (template)
       >>> print (portable_repr (t.render (foo = True)))
       '\\n      I am the onion then head\\n    \\n      And this is the body which should be enclosed by the head/tail\\n    \\n      I am the onion then tail\\n    '
       >>> print (portable_repr (t.render (foo = False)))
       '\\n      And this is the body which should be enclosed by the head/tail\\n    \\n      I am the onion else tail\\n  '
       >>> template = '''
       ...   {% onion foo %}
       ...     {% head %}
       ...       I am the onion then head
       ...     {% else %}
       ...       I am the onion else head
       ...     {% body %}
       ...       And this is the body which should be enclosed by the head/tail
       ...     {% tail %}
       ...       I am the onion then tail
       ...   {% endonion %}
       ... '''.strip ()
       >>> t = env.from_string (template)
       >>> print (portable_repr (t.render (foo = True)))
       '\\n      I am the onion then head\\n    \\n      And this is the body which should be enclosed by the head/tail\\n    \\n      I am the onion then tail\\n  '
       >>> print (portable_repr (t.render (foo = False)))
       '\\n      I am the onion else head\\n    \\n      And this is the body which should be enclosed by the head/tail\\n    '
    """

    tags = set (("onion", ))

    def parse (self, parser) :
        tag    = parser.stream.current
        lineno = next (parser.stream).lineno
        cond   = parser.parse_expression ()
        ### first, seek to the start of the "head" token
        parser.parse_statements (["name:head"], drop_needle = True)
        h_then = parser.parse_statements (["name:else", "name:body"])
        if parser.stream.current.value == "else" :
            next (parser.stream)
            h_else = parser.parse_statements (["name:body"])
        else :
            h_else = None
        next (parser.stream)
        body   = parser.parse_statements (["name:endonion", "name:tail"])
        result = [nodes.If ( cond, h_then, h_else)]
        result.extend (body)
        if parser.stream.current.value == "tail" :
            next (parser.stream)
            t_then = parser.parse_statements (["name:else", "name:endonion"])
            if parser.stream.current.value == "else" :
                next (parser.stream)
                t_else = parser.parse_statements (["name:endonion"])
            else :
                t_else = None
            result.append (nodes.If (cond, t_then, t_else))
        next (parser.stream)
        return result
    # end def parse

# end class Onion

if __name__ != "__main__" :
    JNJ._Export ("Onion")
### __END__ JNJ.Onion
